@@ -168,6 +168,9 @@ process_line (char     *line,
 		fdata->original_path = fdata->full_path + 1;
 	}
 	g_free (name);
+	name = g_filename_from_utf8 (fdata->original_path, -1, NULL, NULL, NULL);
+	if (name)
+		fdata->original_path = name;
 
 	if (fields[1] != NULL)
 		fdata->link = g_strdup (fields[1]);
@@ -203,6 +206,9 @@ add_compress_arg (FrCommand *comm)
 		else
 			fr_process_add_arg (comm->process, "-Z");
 	}
+	else if (is_mime_type (comm->mime_type, "application/x-lrzip-compressed-tar"))
+		fr_process_add_arg (comm->process, "--use-compress-program=lrzip");
+
 	else if (is_mime_type (comm->mime_type, "application/x-lzip-compressed-tar"))
 		fr_process_add_arg (comm->process, "--use-compress-program=lzip");
 
@@ -555,6 +561,26 @@ fr_command_tar_recompress (FrCommand *comm)
 
 		new_name = g_strconcat (c_tar->uncomp_filename, ".Z", NULL);
 	}
+	else if (is_mime_type (comm->mime_type, "application/x-lrzip-compressed-tar")) {
+		fr_process_begin_command (comm->process, "lrzip");
+		fr_process_set_sticky (comm->process, TRUE);
+		fr_process_set_begin_func (comm->process, begin_func__recompress, comm);
+		switch (comm->compression) {
+		case FR_COMPRESSION_VERY_FAST:
+			fr_process_add_arg (comm->process, "-l"); break;
+		case FR_COMPRESSION_FAST:
+			fr_process_add_arg (comm->process, "-g"); break;
+		case FR_COMPRESSION_NORMAL:
+			fr_process_add_arg (comm->process, "-b"); break;
+		case FR_COMPRESSION_MAXIMUM:
+			fr_process_add_arg (comm->process, "-z"); break;
+		}
+		fr_process_add_arg (comm->process, "-o");
+		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
+		fr_process_end_command (comm->process);
+
+		new_name = g_strconcat (c_tar->uncomp_filename, ".lrz", NULL);
+	}
 	else if (is_mime_type (comm->mime_type, "application/x-lzip-compressed-tar")) {
 		fr_process_begin_command (comm->process, "lzip");
 		fr_process_set_sticky (comm->process, TRUE);
@@ -658,7 +684,7 @@ fr_command_tar_recompress (FrCommand *comm)
 		fr_process_add_arg (comm->process, "-l");
 
 		new_name = g_strconcat (c_tar->uncomp_filename, ".7z", NULL);
-		fr_process_add_arg_concat (comm->process, new_name);
+		fr_process_add_arg_concat (comm->process, new_name, NULL);
 
 		fr_process_add_arg (comm->process, c_tar->uncomp_filename);
 		fr_process_end_command (comm->process);
@@ -746,6 +772,17 @@ get_uncompressed_name (FrCommandTar *c_tar,
 		else if (file_extension_is (e_filename, ".tar.Z"))
 			new_name[l - 2] = 0;
 	}
+	else if (is_mime_type (comm->mime_type, "application/x-lrzip-compressed-tar")) {
+		/* X.tlrz     -->  X.tar
+		 * X.tar.lrz  -->  X.tar */
+		if (file_extension_is (e_filename, ".tlrz")) {
+			new_name[l - 3] = 'a';
+			new_name[l - 2] = 'r';
+			new_name[l - 1] = 0;
+		}
+		else if (file_extension_is (e_filename, ".tar.lrz"))
+			new_name[l - 4] = 0;
+	}
 	else if (is_mime_type (comm->mime_type, "application/x-lzip-compressed-tar")) {
 		/* X.tlz     -->  X.tar
 		 * X.tar.lz  -->  X.tar */
@@ -874,6 +911,14 @@ fr_command_tar_uncompress (FrCommand *comm)
 			fr_process_add_arg (comm->process, tmp_name);
 			fr_process_end_command (comm->process);
 		}
+		else if (is_mime_type (comm->mime_type, "application/x-lrzip-compressed-tar")) {
+			fr_process_begin_command (comm->process, "lrzip");
+			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
+			fr_process_add_arg (comm->process, "-f");
+			fr_process_add_arg (comm->process, "-d");
+			fr_process_add_arg (comm->process, tmp_name);
+			fr_process_end_command (comm->process);
+		}
 		else if (is_mime_type (comm->mime_type, "application/x-lzip-compressed-tar")) {
 			fr_process_begin_command (comm->process, "lzip");
 			fr_process_set_begin_func (comm->process, begin_func__uncompress, comm);
@@ -946,6 +991,7 @@ const char *tar_mime_types[] = { "application/x-compressed-tar",
 				 "application/x-bzip-compressed-tar",
 				 "application/x-tar",
 				 "application/x-7z-compressed-tar",
+				 "application/x-lrzip-compressed-tar",
 				 "application/x-lzip-compressed-tar",
 			         "application/x-lzma-compressed-tar",
 			         "application/x-lzop-compressed-tar",
@@ -963,47 +1009,52 @@ fr_command_tar_get_mime_types (FrCommand *comm)
 
 FrCommandCap
 fr_command_tar_get_capabilities (FrCommand  *comm,
-			         const char *mime_type)
+			         const char *mime_type,
+				 gboolean    check_command)
 {
 	FrCommandCap capabilities;
 
 	capabilities = FR_COMMAND_CAN_ARCHIVE_MANY_FILES;
 
 	/* In solaris gtar is present under /usr/sfw/bin */
-	if (! is_program_in_path ("tar") && ! is_program_in_path ("/usr/sfw/bin/gtar"))
+	if (! is_program_available ("tar", check_command) && ! is_program_available ("/usr/sfw/bin/gtar", check_command))
 		return capabilities;
 
 	if (is_mime_type (mime_type, "application/x-tar")) {
 		capabilities |= FR_COMMAND_CAN_READ_WRITE;
 	}
 	else if (is_mime_type (mime_type, "application/x-compressed-tar")) {
-		if (is_program_in_path ("gzip"))
+		if (is_program_available ("gzip", check_command))
 			capabilities |= FR_COMMAND_CAN_READ_WRITE;
 	}
 	else if (is_mime_type (mime_type, "application/x-bzip-compressed-tar")) {
-		if (is_program_in_path ("bzip2"))
+		if (is_program_available ("bzip2", check_command))
 			capabilities |= FR_COMMAND_CAN_READ_WRITE;
 	}
 	else if (is_mime_type (mime_type, "application/x-tarz")) {
-		if (is_program_in_path ("compress") && is_program_in_path ("uncompress"))
+		if (is_program_available ("compress", check_command) && is_program_available ("uncompress", check_command))
 			capabilities |= FR_COMMAND_CAN_READ_WRITE;
-		else if (is_program_in_path ("gzip"))
+		else if (is_program_available ("gzip", check_command))
 			capabilities |= FR_COMMAND_CAN_READ;
 	}
+	else if (is_mime_type (mime_type, "application/x-lrzip-compressed-tar")) {
+		if (is_program_available ("lrzip", check_command))
+			capabilities |= FR_COMMAND_CAN_READ_WRITE;
+	}
 	else if (is_mime_type (mime_type, "application/x-lzip-compressed-tar")) {
-		if (is_program_in_path ("lzip"))
+		if (is_program_available ("lzip", check_command))
 			capabilities |= FR_COMMAND_CAN_READ_WRITE;
 	}
 	else if (is_mime_type (mime_type, "application/x-lzma-compressed-tar")) {
-		if (is_program_in_path ("lzma"))
+		if (is_program_available ("lzma", check_command))
 			capabilities |= FR_COMMAND_CAN_READ_WRITE;
 	}
 	else if (is_mime_type (mime_type, "application/x-xz-compressed-tar")) {
-		if (is_program_in_path ("xz"))
+		if (is_program_available ("xz", check_command))
 			capabilities |= FR_COMMAND_CAN_READ_WRITE;
 	}
 	else if (is_mime_type (mime_type, "application/x-lzop-compressed-tar")) {
-		if (is_program_in_path ("lzop"))
+		if (is_program_available ("lzop", check_command))
 			capabilities |= FR_COMMAND_CAN_READ_WRITE;
 	}
 	else if (is_mime_type (mime_type, "application/x-7z-compressed-tar")) {
@@ -1011,7 +1062,7 @@ fr_command_tar_get_capabilities (FrCommand  *comm,
 		int   i;
 
 		for (i = 0; i < G_N_ELEMENTS (try_command); i++) {
-			if (is_program_in_path (try_command[i])) {
+			if (is_program_available (try_command[i], check_command)) {
 				capabilities |= FR_COMMAND_CAN_WRITE;
 				break;
 			}
@@ -1044,6 +1095,35 @@ fr_command_tar_set_mime_type (FrCommand  *comm,
 }
 
 
+static const char *
+fr_command_tar_get_packages (FrCommand  *comm,
+			     const char *mime_type)
+{
+	if (is_mime_type (mime_type, "application/x-tar"))
+		return PACKAGES ("tar");
+	else if (is_mime_type (mime_type, "application/x-compressed-tar"))
+		return PACKAGES ("tar,gzip");
+	else if (is_mime_type (mime_type, "application/x-bzip-compressed-tar"))
+		return PACKAGES ("tar,bzip2");
+	else if (is_mime_type (mime_type, "application/x-tarz"))
+		return PACKAGES ("tar,gzip,ncompress");
+	else if (is_mime_type (mime_type, "application/x-lrzip-compressed-tar"))
+		return PACKAGES ("tar,lrzip");
+	else if (is_mime_type (mime_type, "application/x-lzip-compressed-tar"))
+		return PACKAGES ("tar,lzip");
+	else if (is_mime_type (mime_type, "application/x-lzma-compressed-tar"))
+		return PACKAGES ("tar,lzma");
+	else if (is_mime_type (mime_type, "application/x-xz-compressed-tar"))
+		return PACKAGES ("tar,xz");
+	else if (is_mime_type (mime_type, "application/x-lzop-compressed-tar"))
+		return PACKAGES ("tar,lzop");
+	else if (is_mime_type (mime_type, "application/x-7z-compressed-tar"))
+		return PACKAGES ("tar,p7zip");
+
+	return NULL;
+}
+
+
 static void
 fr_command_tar_class_init (FrCommandTarClass *class)
 {
@@ -1065,6 +1145,7 @@ fr_command_tar_class_init (FrCommandTarClass *class)
 	afc->set_mime_type    = fr_command_tar_set_mime_type;
 	afc->recompress       = fr_command_tar_recompress;
 	afc->uncompress       = fr_command_tar_uncompress;
+	afc->get_packages     = fr_command_tar_get_packages;
 }
 
 
