@@ -133,10 +133,10 @@ fr_channel_data_read (FrChannelData *channel)
 	char  *line;
 	gsize  length;
 	gsize  terminator_pos;
-	
+
 	channel->status = G_IO_STATUS_NORMAL;
 	g_clear_error (&channel->error);
-		
+
 	while ((channel->status = g_io_channel_read_line (channel->source,
 							  &line,
 							  &length,
@@ -148,7 +148,7 @@ fr_channel_data_read (FrChannelData *channel)
 		if (channel->line_func != NULL)
 			(*channel->line_func) (line, channel->line_data);
 	}
-	
+
 	return channel->status;
 }
 
@@ -157,11 +157,11 @@ static GIOStatus
 fr_channel_data_flush (FrChannelData *channel)
 {
 	GIOStatus status;
-	
+
 	while (((status = fr_channel_data_read (channel)) != G_IO_STATUS_ERROR) && (status != G_IO_STATUS_EOF))
 		/* void */;
 	fr_channel_data_close_source (channel);
-	
+
 	return status;
 }
 
@@ -316,7 +316,7 @@ fr_process_init (FrProcess *process)
 	process->priv->running = FALSE;
 	process->priv->stopping = FALSE;
 	process->restart = FALSE;
-	
+
 	process->priv->current_charset = -1;
 
 	process->priv->use_standard_locale = FALSE;
@@ -632,12 +632,17 @@ fr_process_set_err_line_func (FrProcess *process,
 static gboolean check_child (gpointer data);
 
 
-static void child_setup (gpointer user_data)
+static void
+child_setup (gpointer user_data)
 {
 	FrProcess *process = user_data;
 
 	if (process->priv->use_standard_locale)
 		putenv ("LC_MESSAGES=C");
+
+	/* detach from the tty */
+
+	setsid ();
 }
 
 
@@ -645,12 +650,12 @@ static const char *
 fr_process_get_charset (FrProcess *process)
 {
 	const char *charset = NULL;
-	
+
 	if (process->priv->current_charset >= 0)
 		charset = try_charsets[process->priv->current_charset];
 	else if (g_get_charset (&charset))
 		charset = NULL;
-	
+
 	return charset;
 }
 
@@ -772,7 +777,7 @@ fr_process_set_error (FrProcess       *process,
 		if (gerror != NULL)
 			process->error.gerror = g_error_copy (gerror);
 	}
-}					 	
+}
 
 
 static gint
@@ -826,12 +831,14 @@ check_child (gpointer data)
 				process->error.status = WEXITSTATUS (status);
 			}
 		}
-		else
+		else {
 			process->error.type = FR_PROC_ERROR_EXITED_ABNORMALLY;
+			process->error.status = 255;
+		}
 	}
 
 	process->priv->command_pid = 0;
-	
+
 	if (fr_channel_data_flush (&process->out) == G_IO_STATUS_ERROR) {
 		fr_process_set_error (process, FR_PROC_ERROR_IO_CHANNEL, 0, process->out.error);
 		channel_error = TRUE;
@@ -846,10 +853,10 @@ check_child (gpointer data)
 
 	/**/
 
-	if (channel_error 
+	if (channel_error
 	    && (process->error.type == FR_PROC_ERROR_IO_CHANNEL)
 	    && g_error_matches (process->error.gerror, G_CONVERT_ERROR, G_CONVERT_ERROR_ILLEGAL_SEQUENCE))
-	{	
+	{
 		if (process->priv->current_charset < n_charsets - 1) {
 			/* try with another charset */
 			process->priv->current_charset++;
@@ -857,7 +864,7 @@ check_child (gpointer data)
 			process->restart = TRUE;
 			fr_process_start (process);
 			return FALSE;
-		}		
+		}
 		/*fr_process_set_error (process, FR_PROC_ERROR_NONE, 0, NULL);*/
 		fr_process_set_error (process, FR_PROC_ERROR_BAD_CHARSET, 0, process->error.gerror);
 	}
@@ -870,8 +877,18 @@ check_child (gpointer data)
 
 	/* Execute next command. */
 	if (continue_process) {
-		if (process->error.type != FR_PROC_ERROR_NONE)
+		if (process->error.type != FR_PROC_ERROR_NONE) {
 			allow_sticky_processes_only (process, TRUE);
+#ifdef DEBUG
+			{
+				GList *scan;
+
+				g_print ("** ERROR **\n");
+				for (scan = process->err.raw; scan; scan = scan->next)
+					g_print ("%s\n", (char *)scan->data);
+			}
+#endif
+		}
 
 		if (process->priv->sticky_only) {
 			do {
@@ -903,9 +920,9 @@ check_child (gpointer data)
 
 	if (process->priv->sticky_only) {
 		/* Restore the first error. */
-		fr_process_set_error (process, 
-				      process->priv->first_error.type, 
-				      process->priv->first_error.status, 
+		fr_process_set_error (process,
+				      process->priv->first_error.type,
+				      process->priv->first_error.status,
 				      process->priv->first_error.gerror);
 	}
 
@@ -981,7 +998,7 @@ fr_process_stop_priv (FrProcess *process,
 	if (command_is_sticky (process, process->priv->current_command))
 		allow_sticky_processes_only (process, emit_signal);
 
-	else if (process->term_on_stop)
+	else if (process->term_on_stop && (process->priv->command_pid > 0))
 		kill (process->priv->command_pid, SIGTERM);
 
 	else {
